# -*- coding: ISO-8859-1 -*-
# Copyright (C) 2003-2005 Juan David Ibez Palomar <jdavid@itaapy.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# Import from the Standard Library
import logging

# Import from itools
import itools
from itools import get_abspath
from itools import types
from itools.resources import base, memory
from itools.handlers.Handler import Node as iNode
from itools.xml import namespaces
from itools.gettext import domains
from itools.zope import get_context

# Import from iHotfix
from Products import iHotfix

# Import from ikaaro
from exceptions import Forbidden
from utils import comeback
from LocaleAware import LocaleAware



# Initialize logger
logger = logging.getLogger('update')
logger.setLevel(logging.INFO)
handler = logging.StreamHandler()
logger.addHandler(handler)



class Node(iNode):

    ########################################################################
    # Security
    ########################################################################
    def is_ingroup(self, name):
        """
        Checks wether the authenticated user is in the given group (e.g.
        admins, reviewers) within the context of this resource.
        """
        context = get_context()

        user = context.user
        if user is None:
            return False

        root = context.root
        users = root.get_handler(name).usernames
        if user.name in users:
            return True

        return False


    def is_admin(self):
        return self.is_ingroup('admins')


    def is_reviewer(self):
        return self.is_ingroup('reviewers')


    def is_authenticated(self):
        context = get_context()
        return context.user is not None


    def is_allowed_to_view(self):
        from WorkflowAware import WorkflowAware

        if isinstance(self, WorkflowAware):
            user = get_context().user
            if user is None:
                if self.workflow_state != 'public':
                    return False

        return True


    def is_allowed_to_add(self):
        return get_context().user is not None


    def is_allowed_to_edit(self):
        return get_context().user is not None


    def is_allowed_to_remove(self):
        return get_context().user is not None


    def is_allowed_to_copy(self):
        return get_context().user is not None


    def is_allowed_to_move(self):
        return get_context().user is not None


    def is_allowed_to_translate(self):
        return get_context().user is not None


    def get_method(self, name):
        method = getattr(self, name)

        # Check access
        access = getattr(self, '%s__access__' % name, None)

        # Private methods
        if access is None or access is False:
            return None

        # Public methods
        if access is True:
            return method

        # Is it an aspect?
        if access.im_func(self) is True:
            return method

        return None


    def forbidden_form(self):
        context = get_context()
        if context.user is None:
            here = context.handler
            site_root = here.get_site_root()
            # XXX Add referer
            # XXX aren't redirects supposed to be absolute?
            response = context.response
            response.redirect('%s/;login_form' % here.get_pathto(site_root))
        else:
            return self.gettext('Access forbidden, you are not authorized to'
                                ' access this resource.')


    ########################################################################
    # HTTP
    ########################################################################
    GET__access__ = is_allowed_to_view
    def GET(self):
        context = get_context()
        request, response = context.request, context.response

        method = self.get_firstview()
        # Check access
        if method is None:
            raise Forbidden
        # Calculate the path
        path = request.uri.path.resolve2(';%s' % method)

        response.redirect(path)


    ########################################################################
    # Tree
    ########################################################################
    def get_site_root(self):
        from WebSite import WebSite
        handler = self
        while not isinstance(handler, WebSite):
            handler = handler.parent
        return handler


    ########################################################################
    # Properties
    ########################################################################
    def get_property(self, name, language=None):
        prefix, local_name = types.QName.decode(name)
        namespace = namespaces.get_namespace_by_prefix(prefix)
        schema = namespace.get_attribute_schema(local_name)
        return schema.get('default', '')


    ########################################################################
    # Internationalization
    ########################################################################
    class_domain = 'ikaaro'

    @classmethod
    def select_language(cls, languages):
        accept = get_context().request.accept_language
        return accept.select_language(languages)


    @classmethod
    def gettext(cls, message, language=None):
        gettext = domains.DomainAware.gettext

        if cls.class_domain == 'ikaaro':
            domain_names = ['ikaaro']
        else:
            domain_names = [cls.class_domain, 'ikaaro']

        for domain_name in domain_names:
            if language is None:
                domain = domains.domains[domain_name]
                languages = domain.get_languages()
                language = cls.select_language(languages)

            translation = gettext(message, language, domain=domain_name)
            if translation != message:
                return translation

        return message


    ########################################################################
    # User interface
    ########################################################################
    def get_firstview(self):
        """
        Returns the first allowed object view url, or None if there aren't.
        """
        for name in self.get_views():
            method = self.get_method(name)
            if method is not None:
                return name
        return None


    def get_views(self):
        return []


    def get_subviews(self, name):
        return []



class Handler(itools.handlers.Handler.Handler, Node, domains.DomainAware):

    def get_path_to_icon(self, size=16, from_handler=None):
        if hasattr(self, 'icon%s' % size):
            return ';icon%s' % size
        path_to_icon = getattr(self.__class__, 'class_icon%s' % size)
        if from_handler is None:
            from_handler = self
        return '%sui/%s' % (from_handler.get_pathtoroot(), path_to_icon)


    def get_parent_path(self):
        parent = self.parent
        if parent is None:
            return ''
        elif parent.parent is None:
            return '/'
        return parent.get_abspath()
    
    parent_path = property(get_parent_path, None, None, '')


    def before_commit(self):
        from Root import Root

        root = self.get_root()
        if isinstance(root, Root):
            root.reindex_handler(self)


    ########################################################################
    # The Factory
    ########################################################################
    handler_class_registry = {}

    @classmethod
    def register_handler_class(cls, handler_class):
        cls.handler_class_registry[handler_class.class_id] = handler_class


    @classmethod
    def build_handler(cls, resource, format=None):
        from File import File
        from Folder import Folder

        registry = cls.handler_class_registry
        if format in registry:
            handler_class = registry[format]
        else:
            format = format.split('/')
            if format[0] in registry:
                handler_class = registry[format[0]]
            else:
                # XXX Show a warning message here
                if isinstance(resource, base.File):
                    handler_class = File
                elif isinstance(resource, base.Folder):
                    handler_class = Folder 
                else:
                    raise ValueError, \
                          'Unknown resource type "%s"' % repr(resource)
##        # Check wether the resource is a file and the handler class is a
##        # folder, or viceversa.
##        if isinstance(resource, base.File):
##            if not issubclass(handler_class, File):
##                handler_class = File
##        elif isinstance(resource, base.Folder):
##            if not issubclass(handler_class, Folder):
##                handler_class = Folder

        return handler_class(resource)


    def get_handler_class(self, class_id):
        return self.handler_class_registry[class_id]


    @classmethod
    def new_instance(cls, **kw):
        return cls()


    ########################################################################
    # Properties
    ########################################################################
    def get_property(self, name, language=None):
        metadata = self.metadata
        if metadata is None:
            return Node.get_property(self, name)
        return metadata.get_property(name, language=language)


    def set_property(self, name, value, language=None):
        self.metadata.set_property(name, value, language=language)


    def get_properties_ns(self):
        # XXX Does not works now.
        namespace = {}
        for name in self.metadata.properties.schema:
            value = self.get_property(name)
            namespace[name] = value
        return namespace


    ########################################################################
    # Shorthands
    def get_title(self, language=None):
        return self.get_property('dc:title', language=language)

    title = property(get_title, None, None, '')


    def get_title_or_name(self):
        return self.get_property('dc:title') or self.name

    title_or_name = property(get_title_or_name, None, None, '')


    def get_description(self, language=None):
        desc = self.get_property('dc:description', language=language)
        return desc or ''

    description = property(get_description, None, None, '')


    def get_format(self):
        return self.metadata.get_property('format')

    format = property(get_format, None, None, '')


    def get_owner(self):
        owner = self.metadata.get_property('owner')
        # This guarantees the owner field is always indexed and stored.
        # XXX Maybe another solution should be tried: modify the catalog
        # to support None values (1); be sure the metadata always has
        # an owner value (2) XXX
        if owner is None:
            return ''
        return owner

    owner = property(get_owner, None, None, '')


    def get_mtime(self):
        return self.resource.get_mtime()

    mtime = property(get_mtime, None, None, "")


    def get_mtime_microsecond(self):
        return str(self.get_mtime().microsecond)

    mtime_microsecond = property(get_mtime_microsecond, None, None, '')


    def get_archive_id(self):
        # Versioning
        return self.metadata.get_property('id')


    def get_language(self):
        return self.metadata.get_property('dc:language')

    language = property(get_language, None, None, '')


##    def get_language_name(self):
##        language = self.get_language()
##        return i18n.get_language_name(language)

##    language_name = property(get_language_name, None, None, "")


    ########################################################################
    # Upgrade
    ########################################################################
    def is_uptodate(self):
        object_version = self.metadata.get_property('version')
        class_version = self.class_version
        if object_version is None:
            object_version = class_version

        return class_version == object_version


    def update(self, version=None, *args, **kw):
        # Set zero version if the object does not have a version
        object_version = self.metadata.get_property('version')
        if object_version is None:
            object_version = '00000000'

        # Default version to the current class version
        if version is None:
            version = self.class_version

        # Get all the version numbers
        versions = [ x.split('_')[-1]
                     for x in self.__class__.__dict__.keys()
                     if x.startswith('update_') ]

        # Sort the version numbers
        versions.sort()

        # Filter the versions previous to the current object version
        versions = [ x for x in versions if x > object_version ]

        # Filter the versions next to the given version
        versions = [ x for x in versions if x <= version ]

        # Upgrade
        for version in versions:
            getattr(self, 'update_%s' % version)(*args, **kw)
            logger.info('%s upgraded from %s to %s', self, self.class_version,
                        version)
            self.set_property('version', version)
            self.metadata.save_state()


    ########################################################################
    # HTTP
    ########################################################################
    PUT__access__ = Node.is_authenticated
    def PUT(self):
        context = get_context()
        request, response = context.request, context.response

        # Save the data
        data = request.zope_request['BODY']
        resource = memory.File(data)
        self.load_state(resource)
        self.save_state()
        # Build the response
        response.set_status(204)
        return response.zope_response


    LOCK__access__ = Node.is_authenticated
    def LOCK(self):
        context = get_context()
        request, response = context.request, context.response

        # Lock the resource
        resource = self.resource
        lock, key = resource.lock()
        # Build response
        response.set_header('Content-Type', 'text/xml; charset="utf-8"')
        response.set_header('Lock-Token', 'opaquelocktoken:%s' % key)
        response.set_body(lock.asXML())
        return response.zope_response


    UNLOCK__access__ = Node.is_authenticated
    def UNLOCK(self):
        context = get_context()
        request, response = context.request, context.response

        # Unlock the resource
        resource = self.resource
        key = request.zope_request.get_header('Lock-Token', '')
        resource.unlock(key)

        response.set_status(204)
        return response.zope_response


    ########################################################################
    # User interface
    ########################################################################
    change_content_language__access__ = Node.is_allowed_to_view
    def change_content_language(self, **kw):
        context = get_context()
        request, response = context.request, context.response

        response.set_cookie('content_language', kw['dc:language'])
        response.redirect(request.referer)

##        handler = self.get_version_handler(language=kw['dc:language'])
##        print kw['dc:language'], handler.abspath
##        method_name = request.referer.path[-1].param

##        response.redirect('%s/;%s' % (self.get_pathto(handler), method_name))


    ########################################################################
    # Metadata
    ########################################################################
    def get_metadata(self):
        if self.real_handler is not None:
            return self.real_handler.get_metadata()

        parent = self.parent
        if parent is None:
            return None
        metadata_name = '.%s.metadata' % self.name
        if parent.has_handler(metadata_name):
            return parent.get_handler(metadata_name)
        return None

    metadata = property(get_metadata, None, None, "")


    edit_metadata_form__access__ = Node.is_allowed_to_edit
    edit_metadata_form__label__ = u'Metadata'
    def edit_metadata_form(self):
        context = get_context()
        request = context.request

        # Build the namespace
        namespace = {}
        # Language
        site_root = self.get_site_root()
        languages = site_root.get_property('ikaaro:website_languages')
        default_language = languages[0]
        language = request.get_cookie('content_language') or default_language
        namespace['language'] = language
        # Title
        namespace['title'] = self.get_property('dc:title', language=language)
        # Description
        namespace['description'] = self.get_property('dc:description',
                                                     language=language)

        handler = self.get_handler('/ui/Handler_edit_metadata.xml')
        return handler.stl(namespace)


    edit_metadata__access__ = Node.is_allowed_to_edit
    def edit_metadata(self, **kw):
        context = get_context()
        root = context.root

        language = kw['dc:language']
        if isinstance(self, LocaleAware):
            self.set_property('dc:title', kw['dc:title'])
            self.set_property('dc:description', kw['dc:description'])
        else:
            self.set_property('dc:title', kw['dc:title'], language=language)
            self.set_property('dc:description', kw['dc:description'],
                              language=language)

        # Reindex
        root.reindex_handler(self)

        message = self.gettext('Metadata changed.')
        comeback('edit_metadata_form', message)
